"""The main fritzhome handling class."""
# -*- coding: utf-8 -*-

from __future__ import print_function

import hashlib
import logging
import time
from xml.etree import ElementTree

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from requests import Session

from .errors import InvalidError, LoginError, NotLoggedInError
from .fritzhomedevice import FritzhomeDevice
from .fritzhomedevice import FritzhomeTemplate
from typing import Dict

_LOGGER = logging.getLogger(__name__)


class Fritzhome(object):
    """Fritzhome object to communicate with the device."""

    _sid = None
    _session = None
    _devices: Dict[str, FritzhomeDevice] = None
    _templates: Dict[str, FritzhomeTemplate] = None

    def __init__(self, host, user, password, ssl_verify=True):
        """Create a fritzhome object."""
        self._host = host
        self._user = user
        self._password = password
        self._session = Session()
        self._ssl_verify = ssl_verify

    def _request(self, url, params=None, timeout=10):
        """Send a request with parameters."""
        rsp = self._session.get(
            url, params=params, timeout=timeout, verify=self._ssl_verify
        )
        rsp.raise_for_status()
        return rsp.text.strip()

    def _login_request(self, username=None, secret=None):
        """Send a login request with paramerters."""
        url = self.get_prefixed_host() + "/login_sid.lua?version=2"
        params = {}
        if username:
            params["username"] = username
        if secret:
            params["response"] = secret

        plain = self._request(url, params)
        dom = ElementTree.fromstring(plain)
        sid = dom.findtext("SID")
        blocktime = int(dom.findtext("BlockTime"))
        challenge = dom.findtext("Challenge")

        return (sid, challenge, blocktime)

    def _logout_request(self):
        """Send a logout request."""
        _LOGGER.debug("logout")
        url = self.get_prefixed_host() + "/login_sid.lua"
        params = {"security:command/logout": "1", "sid": self._sid}

        self._request(url, params)

    @staticmethod
    def _create_login_secrete_pbkdf2(challenge, password):
        challenge_parts = challenge.split("$")
        # Extract all necessary values encoded into the challenge
        iter1 = int(challenge_parts[1])
        salt1 = bytes.fromhex(challenge_parts[2])
        iter2 = int(challenge_parts[3])
        salt2 = bytes.fromhex(challenge_parts[4])
        # Hash twice, once with static salt...
        # hash1 = hashlib.pbkdf2_hmac("sha256", password.encode(), salt1, iter1)
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(), length=32, salt=salt1, iterations=iter1
        )
        hash1 = kdf.derive(password.encode())
        # Once with dynamic salt.
        # hash2 = hashlib.pbkdf2_hmac("sha256", hash1, salt2, iter2)
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(), length=32, salt=salt2, iterations=iter2
        )
        hash2 = kdf.derive(hash1)
        return f"{challenge_parts[4]}${hash2.hex()}"

    @staticmethod
    def _create_login_secret_md5(challenge, password):
        """Create a login secret."""
        to_hash = (challenge + "-" + password).encode("UTF-16LE")
        hashed = hashlib.md5(to_hash).hexdigest()
        return "{0}-{1}".format(challenge, hashed)

    def _aha_request(self, cmd, ain=None, param=None, rf=str):
        """Send an AHA request."""
        url = self.get_prefixed_host() + "/webservices/homeautoswitch.lua"

        _LOGGER.debug("self._sid:%s", self._sid)

        if not self._sid:
            raise NotLoggedInError

        params = {"switchcmd": cmd, "sid": self._sid}
        if param:
            params.update(param)
        if ain:
            params["ain"] = ain

        plain = self._request(url, params)
        if plain == "inval":
            raise InvalidError

        if rf == bool:
            return bool(int(plain))
        return rf(plain)

    def login(self):
        """Login and get a valid session ID."""
        (sid, challenge, blocktime) = self._login_request()
        _LOGGER.info("sid:%s, challenge:%s, blocktime:%s", sid, challenge, blocktime)
        if sid == "0000000000000000":
            if blocktime > 0:
                time.sleep(blocktime)
            # PBKDF2 (FRITZ!OS 7.24 or later)
            if challenge.startswith("2$"):
                secret = self._create_login_secrete_pbkdf2(challenge, self._password)
            # fallback to MD5
            else:
                secret = self._create_login_secret_md5(challenge, self._password)
            (sid2, challenge, _) = self._login_request(
                username=self._user, secret=secret
            )
            if sid2 == "0000000000000000":
                _LOGGER.warning("login failed %s", sid2)
                raise LoginError(self._user)
            self._sid = sid2

    def logout(self):
        """Logout."""
        self._logout_request()
        self._sid = None

    def get_prefixed_host(self):
        """Choose the correct protocol prefix for the host.

        Supports three input formats:
        - https://<host>(requests use strict certificate validation by default)
        - http://<host> (unecrypted)
        - <host> (unencrypted)
        """
        host = self._host
        if host.startswith("https://") or host.startswith("http://"):
            return host
        else:
            return "http://" + host

    def update_devices(self, ignore_removed=True):
        """Update the device."""
        _LOGGER.info("Updating Devices ...")
        if self._devices is None:
            self._devices = {}

        device_elements = self.get_device_elements()
        for element in device_elements:
            if element.attrib["identifier"] in self._devices.keys():
                _LOGGER.info(
                    "Updating already existing Device " + element.attrib["identifier"]
                )
                self._devices[element.attrib["identifier"]]._update_from_node(element)
            else:
                _LOGGER.info("Adding new Device " + element.attrib["identifier"])
                device = FritzhomeDevice(self, node=element)
                self._devices[device.ain] = device

        if not ignore_removed:
            for identifier in list(self._devices.keys()):
                if identifier not in [
                    element.attrib["identifier"] for element in device_elements
                ]:
                    _LOGGER.info("Removing no more existing device " + identifier)
                    self._devices.pop(identifier)

        return True

    def _get_listinfo_elements(self, entity_type):
        """Get the DOM elements for the entity list."""
        plain = self._aha_request("get" + entity_type + "listinfos")
        dom = ElementTree.fromstring(plain)
        _LOGGER.debug(dom)
        return dom.findall("*")

    def get_device_elements(self):
        """Get the DOM elements for the device list."""
        return self._get_listinfo_elements("device")

    def get_device_element(self, ain):
        """Get the DOM element for the specified device."""
        elements = self.get_device_elements()
        for element in elements:
            if element.attrib["identifier"] == ain:
                return element
        return None

    def get_devices(self):
        """Get the list of all known devices."""
        return list(self.get_devices_as_dict().values())

    def get_devices_as_dict(self):
        """Get the list of all known devices."""
        if self._devices is None:
            self.update_devices()
        return self._devices

    def get_device_by_ain(self, ain):
        """Return a device specified by the AIN."""
        return self.get_devices_as_dict()[ain]

    def get_device_present(self, ain):
        """Get the device presence."""
        return self._aha_request("getswitchpresent", ain=ain, rf=bool)

    def get_device_name(self, ain):
        """Get the device name."""
        return self._aha_request("getswitchname", ain=ain)

    def get_switch_state(self, ain):
        """Get the switch state."""
        return self._aha_request("getswitchstate", ain=ain, rf=bool)

    def set_switch_state_on(self, ain):
        """Set the switch to on state."""
        return self._aha_request("setswitchon", ain=ain, rf=bool)

    def set_switch_state_off(self, ain):
        """Set the switch to off state."""
        return self._aha_request("setswitchoff", ain=ain, rf=bool)

    def set_switch_state_toggle(self, ain):
        """Toggle the switch state."""
        return self._aha_request("setswitchtoggle", ain=ain, rf=bool)

    def get_switch_power(self, ain):
        """Get the switch power consumption."""
        return self._aha_request("getswitchpower", ain=ain, rf=int)

    def get_switch_energy(self, ain):
        """Get the switch energy."""
        return self._aha_request("getswitchenergy", ain=ain, rf=int)

    def get_temperature(self, ain):
        """Get the device temperature sensor value."""
        return self._aha_request("gettemperature", ain=ain, rf=float) / 10.0

    def _get_temperature(self, ain, name):
        plain = self._aha_request(name, ain=ain, rf=float)
        return (plain - 16) / 2 + 8

    def get_target_temperature(self, ain):
        """Get the thermostate target temperature."""
        return self._get_temperature(ain, "gethkrtsoll")

    def set_target_temperature(self, ain, temperature):
        """Set the thermostate target temperature."""
        temp = int(16 + ((float(temperature) - 8) * 2))

        if temp < min(range(16, 56)):
            temp = 253
        elif temp > max(range(16, 56)):
            temp = 254

        self._aha_request("sethkrtsoll", ain=ain, param={"param": temp})

    def set_window_open(self, ain, seconds):
        """Set the thermostate target temperature."""
        endtimestamp = int(time.time() + seconds)

        self._aha_request(
            "sethkrwindowopen", ain=ain, param={"endtimestamp": endtimestamp}
        )

    def set_boost_mode(self, ain, seconds):
        """Set the thermostate to boost mode."""
        endtimestamp = int(time.time() + seconds)

        self._aha_request("sethkrboost", ain=ain, param={"endtimestamp": endtimestamp})

    def get_comfort_temperature(self, ain):
        """Get the thermostate comfort temperature."""
        return self._get_temperature(ain, "gethkrkomfort")

    def get_eco_temperature(self, ain):
        """Get the thermostate eco temperature."""
        return self._get_temperature(ain, "gethkrabsenk")

    def get_device_statistics(self, ain):
        """Get device statistics."""
        plain = self._aha_request("getbasicdevicestats", ain=ain)
        return plain

    # Lightbulb-related commands

    def set_state_off(self, ain):
        """Set the switch/actuator/lightbulb to on state."""
        self._aha_request("setsimpleonoff", ain=ain, param={"onoff": 0})

    def set_state_on(self, ain):
        """Set the switch/actuator/lightbulb to on state."""
        self._aha_request("setsimpleonoff", ain=ain, param={"onoff": 1})

    def set_state_toggle(self, ain):
        """Toggle the switch/actuator/lightbulb state."""
        self._aha_request("setsimpleonoff", ain=ain, param={"onoff": 2})

    def set_level(self, ain, level):
        """Set level/brightness/height in interval [0,255]."""
        if level < 0:
            level = 0  # 0%
        elif level > 255:
            level = 255  # 100 %

        self._aha_request("setlevel", ain=ain, param={"level": int(level)})

    def set_level_percentage(self, ain, level):
        """Set level/brightness/height in interval [0,100]."""
        if level < 0:
            level = 0
        elif level > 100:
            level = 100

        self._aha_request("setlevelpercentage", ain=ain, param={"level": int(level)})

    def _get_colordefaults(self, ain):
        plain = self._aha_request("getcolordefaults", ain=ain)
        return ElementTree.fromstring(plain)

    def get_colors(self, ain):
        """Get colors (HSV-space) supported by this lightbulb."""
        colordefaults = self._get_colordefaults(ain)
        colors = {}
        for hs in colordefaults.iter("hs"):
            name = hs.find("name").text.strip()
            values = []
            for st in hs.iter("color"):
                values.append((st.get("hue"), st.get("sat"), st.get("val")))
            colors[name] = values
        return colors

    def set_color(self, ain, hsv, duration=0, mapped=True):
        """Set hue and saturation.

        hsv: HUE colorspace element obtained from get_colors()
        duration: Speed of change in seconds, 0 = instant
        """
        params = {
            "hue": int(hsv[0]),
            "saturation": int(hsv[1]),
            "duration": int(duration) * 10,
        }
        if mapped:
            self._aha_request("setcolor", ain=ain, param=params)
        else:
            # undocumented API method for free color selection
            self._aha_request("setunmappedcolor", ain=ain, param=params)

    def get_color_temps(self, ain):
        """Get temperatures supported by this lightbulb."""
        colordefaults = self._get_colordefaults(ain)
        temperatures = []
        for temp in colordefaults.iter("temp"):
            temperatures.append(temp.get("value"))
        return temperatures

    def set_color_temp(self, ain, temperature, duration=0):
        """Set color temperature.

        temperature: temperature element obtained from get_temperatures()
        duration: Speed of change in seconds, 0 = instant
        """
        params = {"temperature": int(temperature), "duration": int(duration) * 10}
        self._aha_request("setcolortemperature", ain=ain, param=params)

    # blinds
    # states: open, close, stop
    def _set_blind_state(self, ain, state):
        self._aha_request("setblind", ain=ain, param={"target": state})

    def set_blind_open(self, ain):
        """Set the blind state to open."""
        self._set_blind_state(ain, "open")

    def set_blind_close(self, ain):
        """Set the blind state to close."""
        self._set_blind_state(ain, "close")

    def set_blind_stop(self, ain):
        """Set the blind state to stop."""
        self._set_blind_state(ain, "stop")

    # Template-related commands

    def has_templates(self):
        """Check if the Fritz!Box supports smarthome templates."""
        plain = self._aha_request("gettemplatelistinfos")
        try:
            ElementTree.fromstring(plain)
        except ElementTree.ParseError:
            return False
        return True

    def update_templates(self, ignore_removed=True):
        """Update the template."""
        _LOGGER.info("Updating Templates ...")
        if self._templates is None:
            self._templates = {}

        template_elements = self.get_template_elements()
        for element in template_elements:
            if element.attrib["identifier"] in self._templates.keys():
                _LOGGER.info(
                    "Updating already existing Template " + element.attrib["identifier"]
                )
                self._templates[element.attrib["identifier"]]._update_from_node(element)
            else:
                _LOGGER.info("Adding new Template " + element.attrib["identifier"])
                template = FritzhomeTemplate(self, node=element)
                self._templates[template.ain] = template

        if not ignore_removed:
            for identifier in list(self._templates.keys()):
                if identifier not in [
                    element.attrib["identifier"] for element in template_elements
                ]:
                    _LOGGER.info("Removing no more existing template " + identifier)
                    self._templates.pop(identifier)

        return True

    def get_template_elements(self):
        """Get the DOM elements for the template list."""
        return self._get_listinfo_elements("template")

    def get_templates(self):
        """Get the list of all known templates."""
        return list(self.get_templates_as_dict().values())

    def get_templates_as_dict(self):
        """Get the list of all known templates."""
        if self._templates is None:
            self.update_templates()
        return self._templates

    def get_template_by_ain(self, ain):
        """Return a template specified by the AIN."""
        return self.get_templates_as_dict()[ain]

    def apply_template(self, ain):
        """Appliy a template."""
        self._aha_request("applytemplate", ain=ain)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock, patch

from pyfritzhome import Fritzhome, FritzhomeDevice
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceThermostat(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_device_hkr_fw_03_50(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_fw_03_50"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.present
        assert device.device_lock is None
        assert device.lock is None
        assert device.error_code is None
        assert device.battery_low is None
        assert device.supported_features == [
            FritzhomeDeviceFeatures.THERMOSTAT,
            FritzhomeDeviceFeatures.TEMPERATURE,
        ]

    def test_device_hkr_fw_03_54(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_fw_03_54"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("23456")
        assert device.present
        assert device.supported_features == [
            FritzhomeDeviceFeatures.THERMOSTAT,
            FritzhomeDeviceFeatures.TEMPERATURE,
        ]

    def test_get_target_temperature(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_fritzos_7"),
            "38",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        assert device.get_target_temperature() == 19.0
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "12345", "switchcmd": "gethkrtsoll", "sid": "0000001"},
        )

    def test_get_eco_temperature(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_fritzos_7"),
            "40",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        assert device.get_eco_temperature() == 20.0
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "12345", "switchcmd": "gethkrabsenk", "sid": "0000001"},
        )

    def test_get_comfort_temperature(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_fritzos_7"),
            "41",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        assert device.get_comfort_temperature() == 20.5
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "12345", "switchcmd": "gethkrkomfort", "sid": "0000001"},
        )

    def test_hkr_without_temperature_values(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_no_temp_values"),
        ]

        element = self.fritz.get_device_element("11960 0071472")
        device = FritzhomeDevice(node=element)

        assert device.ain == "11960 0071472"
        assert device.offset is None
        assert device.temperature is None

    def test_hkr_get_state_on(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_state_on"),
            Helper.response("thermostat/device_hkr_state_on"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.get_hkr_state() == "on"

    def test_hkr_get_state_off(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_state_off"),
            Helper.response("thermostat/device_hkr_state_off"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.get_hkr_state() == "off"

    def test_hkr_get_state_eco(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_state_eco"),
            Helper.response("thermostat/device_hkr_state_eco"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.get_hkr_state() == "eco"

    def test_hkr_get_state_comfort(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_state_comfort"),
            Helper.response("thermostat/device_hkr_state_comfort"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.get_hkr_state() == "comfort"

    def test_hkr_get_state_manual(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_state_manual"),
            Helper.response("thermostat/device_hkr_state_manual"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.get_hkr_state() == "manual"

    def test_hkr_set_state_on(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_state_manual"),
            Helper.response("thermostat/device_hkr_state_manual"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        device.set_hkr_state("on")
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "ain": "12345",
                "switchcmd": "sethkrtsoll",
                "param": 254,
                "sid": "0000001",
            },
        )

    def test_hkr_set_state_off(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_state_manual"),
            Helper.response("thermostat/device_hkr_state_manual"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        device.set_hkr_state("off")
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "ain": "12345",
                "switchcmd": "sethkrtsoll",
                "param": 253,
                "sid": "0000001",
            },
        )

    def test_hkr_battery_level(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.battery_level == 70

    def test_hkr_window_open(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert not device.window_open

    @patch("time.time", MagicMock(return_value=1704630800))
    def test_hkr_boost_mode(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7_57")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345 6789012")
        assert device.boost_active is True
        assert device.boost_active_endtime == 42

    def test_hkr_adaptive_heating(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7_57")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345 6789012")
        assert device.adaptive_heating_active is True
        assert device.adaptive_heating_running is False

    def test_hkr_summer_active(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.summer_active

    def test_hkr_holiday_active(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert not device.holiday_active

    def test_hkr_nextchange_endperiod(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.nextchange_endperiod == 1538341200

    def test_hkr_nextchange_temperature(self):
        self.mock.side_effect = [Helper.response("thermostat/device_hkr_fritzos_7")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")
        assert device.nextchange_temperature == 21.0



"""The humidity device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceHumidity(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    rel_humidity = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_humidity_sensor:
            self._update_humidity_from_node(node)

    # Humidity
    @property
    def has_humidity_sensor(self):
        """Check if the device has humidity function."""
        return self._has_feature(FritzhomeDeviceFeatures.HUMIDITY)

    def _update_humidity_from_node(self, node):
        _LOGGER.debug("update humidity device")
        humidity_element = node.find("humidity")
        try:
            self.rel_humidity = self.get_node_value_as_int(
                humidity_element, "rel_humidity"
            )
        except ValueError:
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceAlarm(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_device_alert_on(self):
        self.mock.side_effect = [
            Helper.response("alarm/device_alert_on"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("05333 0077045-1")
        assert device.present
        assert device.alert_state
        assert device.supported_features == [
            FritzhomeDeviceFeatures.ALARM,
            FritzhomeDeviceFeatures.HANFUN_UNIT,
        ]

    def test_device_alert_off(self):
        self.mock.side_effect = [
            Helper.response("alarm/device_alert_off"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("05333 0077045-2")
        assert device.present
        assert not device.alert_state

    def test_device_alert_no_alertstate(self):
        self.mock.side_effect = [
            Helper.response("alarm/device_alert_no_alertstate"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("05333 0077045-3")
        assert device.present
        assert device.alert_state is None

    def test_magenta_smoke_alarm(self):
        self.mock.side_effect = [
            Helper.response("alarm/device_magenta_smoke_alarm"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("11324 0244498-1")
        assert device.present
        assert device.alert_state is None



"""Project specific exceptions."""


class LoginError(Exception):
    """The LoginError Exception."""

    def __init__(self, user):
        """Initialize the an loginError."""
        self.user = user

    def __str__(self):
        """Return the error."""
        return 'login for user="{}" failed'.format(self.user)


class NotLoggedInError(Exception):
    """The NotLoggedInError Exception."""

    def __str__(self):
        """Return the error."""
        return "not logged in, login before doing any requests."


class InvalidError(Exception):
    """The InvalidError Exception."""

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceLightBulb(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_device_init(self):
        self.mock.side_effect = [
            Helper.response("lightbulb/device_FritzDECT500_34_12_16")
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        assert device.ain == "12345"
        assert device.fw_version == "34.10.16.16.009"
        assert device.present  # Lightbulb has power and is connected
        assert device.supported_features == [FritzhomeDeviceFeatures.HANFUN_DEVICE]

        # Get sub-device
        device = self.fritz.get_device_by_ain("12345-1")
        assert device.has_lightbulb
        assert device.has_level
        assert device.has_color
        assert device.state  # Lightbulb is switched on
        assert device.color_mode == "1"
        assert device.supported_color_mode == "5"
        assert device.hue == 358
        assert device.saturation == 180
        assert device.color_temp is None
        assert device.name == "FRITZ!DECT 500 Büro"
        assert device.supported_features == [
            FritzhomeDeviceFeatures.LIGHTBULB,
            FritzhomeDeviceFeatures.HANFUN_UNIT,
            FritzhomeDeviceFeatures.SWITCHABLE,
            FritzhomeDeviceFeatures.LEVEL,
            FritzhomeDeviceFeatures.COLOR,
        ]

    def test_device_init_non_color_bulb(self):
        self.mock.side_effect = [
            Helper.response("lightbulb/device_Telekom_Magenta_NonColorBulb")
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12701 0072784")

        assert device.ain == "12701 0072784"
        assert device.fw_version == "34.09.15.16.018"
        assert device.present  # Lightbulb has power and is connected

        # Get sub-device
        device = self.fritz.get_device_by_ain("12701 0072784-1")
        assert device.has_lightbulb
        assert device.has_level
        assert device.state  # Lightbulb is switched on
        assert device.name == "Telekom White Dimmable Bulb"

    def test_device_init_color_temp_mode(self):
        self.mock.side_effect = [
            Helper.response("lightbulb/device_FritzDECT500_34_12_16_color_temp_mode")
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        assert device.ain == "12345"
        assert device.fw_version == "34.10.16.16.009"
        assert device.present  # Lightbulb has power and is connected

        # Get sub-device
        device = self.fritz.get_device_by_ain("12345-1")
        assert device.has_lightbulb
        assert device.state  # Lightbulb is switched on
        assert device.color_mode == "4"
        assert device.supported_color_mode == "5"
        assert device.hue is None
        assert device.saturation is None
        assert device.color_temp == 2800
        assert device.name == "FRITZ!DECT 500 Büro"

    def test_get_colors(self):
        self.mock.side_effect = [
            Helper.response("lightbulb/device_FritzDECT500_34_12_16")
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345-1")

        self.mock.side_effect = [
            Helper.response("lightbulb/getcolors_FritzDECT500_34_12_16")
        ]

        colors = device.get_colors()
        # fmt: off
        expected_colors = {
            'Rot': [
                ('358', '180', '230'),
                ('358', '112', '237'),
                ('358', '54', '245')
                ],
            'Orange': [
                ('35', '214', '255'),
                ('35', '140', '255'),
                ('35', '72', '255')
                ],
            'Gelb': [
                ('52', '153', '252'),
                ('52', '102', '252'),
                ('52', '51', '255')
                ],
            'Grasgrün': [
                ('92', '123', '248'),
                ('92', '79', '250'),
                ('92', '38', '252')
                ],
            'Grün': [
                ('120', '160', '220'),
                ('120', '82', '232'),
                ('120', '38', '242')
                ],
            'Türkis': [
                ('160', '145', '235'),
                ('160', '84', '242'),
                ('160', '41', '248')
                ],
            'Cyan': [
                ('195', '179', '255'),
                ('195', '118', '255'),
                ('195', '59', '255')
                ],
            'Himmelblau': [
                ('212', '169', '252'),
                ('212', '110', '252'),
                ('212', '56', '255')
                ],
            'Blau': [
                ('225', '204', '255'),
                ('225', '135', '255'),
                ('225', '67', '255')
                ],
            'Violett': [
                ('266', '169', '250'),
                ('266', '110', '250'),
                ('266', '54', '252')
                ],
            'Magenta': [
                ('296', '140', '250'),
                ('296', '92', '252'),
                ('296', '46', '255')
                ],
            'Pink': [
                ('335', '180', '255'),
                ('335', '107', '248'),
                ('335', '51', '250')
                ]
            }
        # fmt: on
        assert colors == expected_colors

    def test_get_color_temps(self):
        self.mock.side_effect = [
            Helper.response("lightbulb/device_FritzDECT500_34_12_16")
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345-1")

        self.mock.side_effect = [
            Helper.response("lightbulb/getcolors_FritzDECT500_34_12_16")
        ]

        temps = device.get_color_temps()
        expected_temps = [
            "2700",
            "3000",
            "3400",
            "3800",
            "4200",
            "4700",
            "5300",
            "5900",
            "6500",
        ]
        assert temps == expected_temps

    def test_set_color(self):
        self.mock.side_effect = [
            Helper.response("lightbulb/device_FritzDECT500_34_12_16"),
            "1",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345-1")

        device.set_color(["180", "200"], 0)

        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "switchcmd": "setcolor",
                "sid": "0000001",
                "hue": 180,
                "saturation": 200,
                "duration": 0,
                "ain": "12345-1",
            },
        )

    def test_set_unmapped_color(self):
        self.mock.side_effect = [
            Helper.response("lightbulb/device_FritzDECT500_34_12_16"),
            "1",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345-1")

        device.set_unmapped_color(["180", "200"], 0)

        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "switchcmd": "setunmappedcolor",
                "sid": "0000001",
                "hue": 180,
                "saturation": 200,
                "duration": 0,
                "ain": "12345-1",
            },
        )



"""The powermeter device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDevicePowermeter(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    power = None
    energy = None
    voltage = None
    current = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_powermeter:
            self._update_powermeter_from_node(node)

    # Power Meter
    @property
    def has_powermeter(self):
        """Check if the device has powermeter function."""
        return self._has_feature(FritzhomeDeviceFeatures.POWER_METER)

    def _update_powermeter_from_node(self, node):
        _LOGGER.debug("update powermeter device")
        val = node.find("powermeter")
        self.power = int(val.findtext("power"))
        self.energy = int(val.findtext("energy"))
        try:
            self.voltage = int(val.findtext("voltage"))
        except Exception:
            pass

        if (
            isinstance(self.power, int)
            and isinstance(self.voltage, int)
            and self.voltage > 0
        ):
            self.current = self.power / self.voltage * 1000
        else:
            self.current = None

    def get_switch_power(self):
        """Get the switch state."""
        return self._fritz.get_switch_power(self.ain)

    def get_switch_energy(self):
        """Get the switch energy."""
        return self._fritz.get_switch_energy(self.ain)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures
from pyfritzhome.devicetypes.fritzhomeentitybase import FritzhomeEntityBase

from .helper import Helper


class TestFritzhomeDeviceBase(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_device_init(self):
        self.mock.side_effect = [Helper.response("base/device_list")]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        assert device.ain == "08761 0000434"
        assert device.fw_version == "03.33"
        assert device.present
        assert device.has_switch
        assert device.has_temperature_sensor
        assert device.has_powermeter
        assert device.supported_features == [
            FritzhomeDeviceFeatures.POWER_METER,
            FritzhomeDeviceFeatures.TEMPERATURE,
            FritzhomeDeviceFeatures.SWITCH,
        ]

    def test_device_init_present_false(self):
        self.mock.side_effect = [
            Helper.response("base/device_not_present"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("11960 0089208")

        assert device.ain == "11960 0089208"
        assert not device.present

    def test_device_init_no_devicelock_element(self):
        self.mock.side_effect = [
            Helper.response("base/device_no_devicelock_element"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0373130")

        assert device.ain == "08761 0373130"
        assert device.present

    def test_device_umlaut(self):
        self.mock.side_effect = [
            Helper.response("base/device_with_umlaut_in_name"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0373130")

        assert device.ain == "08761 0373130"
        assert device.name == "äöü"

    def test_device_update(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_list_battery_ok"),
            Helper.response("thermostat/device_list_battery_low"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("11959 0171328")
        assert not device.battery_low
        device.update()
        assert device.battery_low

    def test_get_device_present(self):
        self.mock.side_effect = [
            Helper.response("base/device_list"),
            "1",
            "0",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        assert device.get_present()
        assert not device.get_present()
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "08761 0000434", "switchcmd": "getswitchpresent", "sid": "0000001"},
        )

    def test_device_removed(self):
        self.mock.side_effect = [
            Helper.response("base/device_list"),
            Helper.response("base/device_list_removed_device"),
            Helper.response("base/device_list_removed_device"),
        ]

        self.fritz.update_devices()
        assert len(self.fritz.get_devices()) == 5
        self.fritz.update_devices()
        assert len(self.fritz.get_devices()) == 5
        self.fritz.update_devices(ignore_removed=False)
        assert len(self.fritz.get_devices()) == 4

    def test_device_and_unit_id(self):
        device = FritzhomeEntityBase()

        device.ain = "11630 0114733"
        assert device.device_and_unit_id == ("11630 0114733", None)

        device.ain = "11630 0114733-1"
        assert device.device_and_unit_id == ("11630 0114733", "1")

        device.ain = "ZA4C1380C30E07AB1"
        assert device.device_and_unit_id == ("ZA4C1380C30E07AB1", None)

        device.ain = "ZA4C1380C30E07AB101"
        assert device.device_and_unit_id == ("ZA4C1380C30E07AB1", "01")

        device.ain = "grp303E4F-3F7D9BE07"
        assert device.device_and_unit_id == ("grp303E4F-3F7D9BE07", None)

        device.ain = "tmp816271-3F6EB615E"
        assert device.device_and_unit_id == ("tmp816271-3F6EB615E", None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceButton(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_button_fritzdect440(self):
        self.mock.side_effect = [
            Helper.response("button/device_button_fritzdect440"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345 0000001")
        assert device.present
        assert device.alert_state is None
        assert device.has_temperature_sensor
        assert device.has_button
        assert not device.battery_low
        assert device.battery_level == 100
        assert not device.tx_busy
        assert device.supported_features == [
            FritzhomeDeviceFeatures.BUTTON,
            FritzhomeDeviceFeatures.TEMPERATURE,
        ]

        button = device.get_button_by_ain("12345 0000001-1")
        assert button.name == "Taster Wohnzimmer: Oben rechts"
        assert button.last_pressed == 1608557681

        button = device.get_button_by_ain("12345 0000001-2")
        assert button.name == "Taster Wohnzimmer: Unten rechts"
        assert button.last_pressed == 1608557682

    def test_button_fritzdect440_humidity(self):
        self.mock.side_effect = [
            Helper.response("button/device_button_fritzdect440_fw_05_10"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345 0000002")
        assert device.present
        assert device.rel_humidity == 44



"""The switch device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceSwitch(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    switch_state = None
    switch_mode = None
    lock = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_switch:
            self._update_switch_from_node(node)

    # Switch
    @property
    def has_switch(self):
        """Check if the device has switch function."""
        if self._has_feature(FritzhomeDeviceFeatures.SWITCH):
            # for AVM plugs like FRITZ!DECT 200 and FRITZ!DECT 210
            return True
        if self._has_feature(
            FritzhomeDeviceFeatures.SWITCHABLE
        ) and not self._has_feature(FritzhomeDeviceFeatures.LIGHTBULB):
            # for HAN-FUN plugs
            return True
        return False

    def _update_switch_from_node(self, node):
        _LOGGER.debug("update switch device")
        if self._has_feature(FritzhomeDeviceFeatures.SWITCH):
            val = node.find("switch")
            try:
                self.switch_state = self.get_node_value_as_int_as_bool(val, "state")
            except Exception:
                self.switch_state = None
            self.switch_mode = self.get_node_value(val, "mode")
            try:
                self.lock = self.get_node_value_as_int_as_bool(val, "lock")
            except Exception:
                self.lock = None

            # optional value
            try:
                self.device_lock = self.get_node_value_as_int_as_bool(val, "devicelock")
            except Exception:
                pass
        else:
            val = node.find("simpleonoff")
            try:
                self.switch_state = self.get_node_value_as_int_as_bool(val, "state")
            except Exception:
                self.switch_state = None

    def get_switch_state(self):
        """Get the switch state."""
        return self._fritz.get_switch_state(self.ain)

    def set_switch_state_on(self):
        """Set the switch state to on."""
        return self._fritz.set_switch_state_on(self.ain)

    def set_switch_state_off(self):
        """Set the switch state to off."""
        return self._fritz.set_switch_state_off(self.ain)

    def set_switch_state_toggle(self):
        """Toggle the switch state."""
        return self._fritz.set_switch_state_toggle(self.ain)



"""The repeater device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceRepeater(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

    # Repeater
    @property
    def has_repeater(self):
        """Check if the device has repeater function."""
        return self._has_feature(FritzhomeDeviceFeatures.DECT_REPEATER)



"""Init file for pyfritzhome."""

from .errors import InvalidError, LoginError, NotLoggedInError
from .fritzhome import Fritzhome
from .fritzhomedevice import FritzhomeDevice

__all__ = (
    "Fritzhome",
    "FritzhomeDevice",
    "InvalidError",
    "LoginError",
    "NotLoggedInError",
)



"""Toplevel device for pyfritzhome."""

# -*- coding: utf-8 -*-

from .devicetypes import FritzhomeTemplate  # noqa: F401
from .devicetypes import (
    FritzhomeDeviceAlarm,
    FritzhomeDeviceBlind,
    FritzhomeDeviceButton,
    FritzhomeDeviceHumidity,
    FritzhomeDeviceLevel,
    FritzhomeDeviceLightBulb,
    FritzhomeDevicePowermeter,
    FritzhomeDeviceRepeater,
    FritzhomeDeviceSwitch,
    FritzhomeDeviceTemperature,
    FritzhomeDeviceThermostat,
)


class FritzhomeDevice(
    FritzhomeDeviceAlarm,
    FritzhomeDeviceBlind,
    FritzhomeDeviceButton,
    FritzhomeDeviceHumidity,
    FritzhomeDeviceLevel,
    FritzhomeDeviceLightBulb,
    FritzhomeDevicePowermeter,
    FritzhomeDeviceRepeater,
    FritzhomeDeviceSwitch,
    FritzhomeDeviceTemperature,
    FritzhomeDeviceThermostat,
):
    """The Fritzhome Device class."""

    def __init__(self, fritz=None, node=None):
        """Create a device object."""
        super().__init__(fritz, node)

    def _update_from_node(self, node):
        super()._update_from_node(node)



"""The entity base class."""
# -*- coding: utf-8 -*-

from __future__ import print_function
from abc import ABC


import logging
from xml.etree import ElementTree
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeEntityBase(ABC):
    """The Fritzhome Entity class."""

    _fritz = None
    ain: str = None
    _functionsbitmask = None
    supported_features = None

    def __init__(self, fritz=None, node=None):
        """Create an entity base object."""
        if fritz is not None:
            self._fritz = fritz
        if node is not None:
            self._update_from_node(node)

    def __repr__(self):
        """Return a string."""
        return "{ain} {name}".format(
            ain=self.ain,
            name=self.name,
        )

    def _has_feature(self, feature: FritzhomeDeviceFeatures) -> bool:
        return feature in FritzhomeDeviceFeatures(self._functionsbitmask)

    def _update_from_node(self, node):
        _LOGGER.debug(ElementTree.tostring(node))
        self.ain = node.attrib["identifier"]
        self._functionsbitmask = int(node.attrib["functionbitmask"])

        self.name = node.findtext("name").strip()

        self.supported_features = []
        for feature in FritzhomeDeviceFeatures:
            if self._has_feature(feature):
                self.supported_features.append(feature)

    @property
    def device_and_unit_id(self):
        """Get the device and possible unit id."""
        if self.ain.startswith("tmp") or self.ain.startswith("grp"):
            return (self.ain, None)
        elif self.ain.startswith("Z") and len(self.ain) == 19:
            return (self.ain[0:17], self.ain[17:])
        elif "-" in self.ain:
            return tuple(self.ain.split("-"))
        return (self.ain, None)

    # XML Helpers

    def get_node_value(self, elem, node):
        """Get the node value."""
        return elem.findtext(node)

    def get_node_value_as_int(self, elem, node) -> int:
        """Get the node value as integer."""
        return int(self.get_node_value(elem, node))

    def get_node_value_as_int_as_bool(self, elem, node) -> bool:
        """Get the node value as boolean."""
        return bool(self.get_node_value_as_int(elem, node))

    def get_temp_from_node(self, elem, node):
        """Get the node temp value as float."""
        return float(self.get_node_value(elem, node)) / 2



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceThermostat(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_device_alert_on(self):
        self.mock.side_effect = [
            Helper.response("groups/device_list_thermostat"),
        ]

        self.fritz.update_devices()
        group = self.fritz.get_device_by_ain("grp303E4F-3F7D9BE07")
        assert group.has_thermostat
        assert group.is_group
        assert group.group_members == ["16", "17"]
        assert group.supported_features == [FritzhomeDeviceFeatures.THERMOSTAT]
        assert group.adaptive_heating_active is True
        assert group.adaptive_heating_running is False
        assert group.boost_active is False
        assert group.window_open is False
        assert group.boost_active_endtime == 0
        assert group.window_open_endtime == 0



"""The template class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomeentitybase import FritzhomeEntityBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeTemplate(FritzhomeEntityBase):
    """The Fritzhome Template class."""

    devices = None
    features = None
    apply_hkr_summer = None
    apply_hkr_temperature = None
    apply_hkr_holidays = None
    apply_hkr_time_table = None
    apply_relay_manual = None
    apply_relay_automatic = None
    apply_level = None
    apply_color = None
    apply_dialhelper = None

    def _update_from_node(self, node):
        _LOGGER.debug("update template")
        super()._update_from_node(node)

        self.features = FritzhomeDeviceFeatures(self._functionsbitmask)

        applymask = node.find("applymask")
        self.apply_hkr_summer = applymask.find("hkr_summer") is not None
        self.apply_hkr_temperature = applymask.find("hkr_temperature") is not None
        self.apply_hkr_holidays = applymask.find("hkr_holidays") is not None
        self.apply_hkr_time_table = applymask.find("hkr_time_table") is not None
        self.apply_relay_manual = applymask.find("relay_manual") is not None
        self.apply_relay_automatic = applymask.find("relay_automatic") is not None
        self.apply_level = applymask.find("level") is not None
        self.apply_color = applymask.find("color") is not None
        self.apply_dialhelper = applymask.find("dialhelper") is not None

        self.devices = []
        for device in node.find("devices").findall("device"):
            self.devices.append(device.attrib["identifier"])



"""The blind device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceBlind(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    endpositionsset = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_blind:
            self._update_blind_from_node(node)

    # Blind
    @property
    def has_blind(self):
        """Check if the device has blind function."""
        return self._has_feature(FritzhomeDeviceFeatures.BLIND)

    def _update_blind_from_node(self, node):
        _LOGGER.debug("update blind device")
        blind_element = node.find("blind")
        try:
            self.endpositionsset = self.get_node_value_as_int_as_bool(
                blind_element, "endpositionsset"
            )
        except Exception:
            pass

    def set_blind_open(self):
        """Open the blind."""
        self._fritz.set_blind_open(self.ain)

    def set_blind_close(self):
        """Close the blind."""
        self._fritz.set_blind_close(self.ain)

    def set_blind_stop(self):
        """Stop the blind."""
        self._fritz.set_blind_stop(self.ain)



#!/usr/bin/env python
"""A simple CLI tool."""
# -*- coding: utf-8 -*-

from __future__ import print_function
import logging
import argparse

try:
    from version import __version__
except ImportError:
    __version__ = "dev"

from pyfritzhome import Fritzhome

_LOGGER = logging.getLogger(__name__)


def list_all(fritz, args):
    """Command that prints all device information."""
    devices = fritz.get_devices()

    for device in devices:
        print("#" * 30)
        print("name=%s" % device.name)
        print("  ain=%s" % device.ain)
        print("  id=%s" % device.identifier)
        print("  productname=%s" % device.productname)
        print("  manufacturer=%s" % device.manufacturer)
        print("  present=%s" % device.present)
        print("  lock=%s" % device.lock)
        print("  devicelock=%s" % device.device_lock)
        print("  is_group=%s" % device.is_group)
        if device.is_group:
            print("  group_members=%s" % device.group_members)

        if device.present is False:
            continue

        if device.has_switch:
            print(" Switch:")
            print("  switch_state=%s" % device.switch_state)
        if device.has_powermeter:
            print(" Powermeter:")
            print("  power=%s" % device.power)
            print("  energy=%s" % device.energy)
            print("  voltage=%s" % device.voltage)
        if device.has_temperature_sensor:
            print(" Temperature:")
            print("  temperature=%s" % device.temperature)
            print("  offset=%s" % device.offset)
        if device.has_thermostat:
            print(" Thermostat:")
            print("  battery_low=%s" % device.battery_low)
            print("  battery_level=%s" % device.battery_level)
            print("  actual=%s" % device.actual_temperature)
            print("  target=%s" % device.target_temperature)
            print("  comfort=%s" % device.comfort_temperature)
            print("  eco=%s" % device.eco_temperature)
            print("  window=%s" % device.window_open)
            print("  window_until=%s" % device.window_open_endtime)
            print("  boost=%s" % device.boost_active)
            print("  boost_until=%s" % device.boost_active_endtime)
            print("  adaptive_heating_running=%s" % device.adaptive_heating_running)
            print("  summer=%s" % device.summer_active)
            print("  holiday=%s" % device.holiday_active)
        if device.has_alarm:
            print(" Alert:")
            print("  alert=%s" % device.alert_state)
        if device.has_lightbulb:
            print(" Light bulb:")
            print("  state=%s" % ("Off" if device.state == 0 else "On"))
            if device.has_level:
                print("  level=%s" % device.level)
            if device.has_color:
                print("  hue=%s" % device.hue)
                print("  saturation=%s" % device.saturation)
        if device.has_blind:
            print(" Blind:")
            print("  level=%s" % device.level)
            print("  levelpercentage=%s" % device.levelpercentage)
            print("  endpositionset=%s" % device.endpositionsset)


def device_name(fritz, args):
    """Command that prints the device name."""
    print(fritz.get_device_name(args.ain))


def device_presence(fritz, args):
    """Command that prints the device presence."""
    print(int(fritz.get_device_present(args.ain)))


def device_statistics(fritz, args):
    """Command that prints the device statistics."""
    stats = fritz.get_device_statistics(args.ain)
    print(stats)


def blind_set_open(fritz, args):
    """Command to open the blinds."""
    fritz.set_blind_open(args.ain)


def blind_set_close(fritz, args):
    """Command close the blinds."""
    fritz.set_blind_close(args.ain)


def blind_set_level_percentage(fritz, args):
    """Command that sets the blind level as percentage."""
    fritz.set_level_percentage(args.ain, args.level)


def thermostat_set_target_temperature(fritz, args):
    """Command that sets the thermostat temperature."""
    fritz.set_target_temperature(args.ain, args.temperature)


def thermostat_set_window_open(fritz, args):
    """Command that sets the thermostats window state."""
    fritz.set_window_open(args.ain, args.timespan)


def thermostat_set_boost_mode(fritz, args):
    """Command that sets the thermostats into boost mode."""
    fritz.set_boost_mode(args.ain, args.timespan)


def switch_get(fritz, args):
    """Command that get the device switch state."""
    print(fritz.get_switch_state(args.ain))


def switch_on(fritz, args):
    """Command that set the device switch state to on."""
    fritz.set_switch_state_on(args.ain)


def switch_off(fritz, args):
    """Command that set the device switch state to off."""
    fritz.set_switch_state_off(args.ain)


def switch_toggle(fritz, args):
    """Command that toggles the device switch state."""
    fritz.set_switch_state_toggle(args.ain)


def list_templates(fritz, args):
    """Command that prints all template information."""
    templates = fritz.get_templates()
    devices = fritz.get_devices_as_dict()

    for template in templates:
        print("#" * 30)
        print("name=%s" % template.name)
        print("  ain=%s" % template.ain)

        print(" Apply:")
        print("  hkr_summer=%s" % template.apply_hkr_summer)
        print("  hkr_temperature=%s" % template.apply_hkr_temperature)
        print("  hkr_holidays=%s" % template.apply_hkr_holidays)
        print("  hkr_time_table=%s" % template.apply_hkr_time_table)
        print("  relay_manual=%s" % template.apply_relay_manual)
        print("  relay_automatic=%s" % template.apply_relay_automatic)
        print("  level=%s" % template.apply_level)
        print("  color=%s" % template.apply_color)
        print("  dialhelper=%s" % template.apply_dialhelper)

        print(" Devices:")
        for device_id in template.devices:
            print("  %s=%s" % (device_id, devices[device_id].name))


def template_apply(fritz, args):
    """Command that applies a template."""
    fritz.apply_template(args.ain)


def main(args=None):
    """Enter the main function of the CLI tool."""
    parser = argparse.ArgumentParser(description="Fritz!Box Smarthome CLI tool.")
    parser.add_argument(
        "-v", action="store_true", dest="verbose", help="be more verbose"
    )
    parser.add_argument(
        "-f",
        "--fritzbox",
        type=str,
        dest="host",
        help="Fritz!Box IP address",
        default="fritz.box",
    )
    parser.add_argument("-u", "--user", type=str, dest="user", help="Username")
    parser.add_argument("-p", "--password", type=str, dest="password", help="Username")
    parser.add_argument(
        "-a",
        "--ain",
        type=str,
        dest="ain",
        help="Actor/Template Identification",
        default=None,
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version="{version}".format(version=__version__),
        help="Print version",
    )

    _sub = parser.add_subparsers(title="Commands")

    # list all devices
    subparser = _sub.add_parser("list", help="List all available devices")
    subparser.set_defaults(func=list_all)

    # device
    subparser = _sub.add_parser("device", help="Device/Actor commands")
    _sub_switch = subparser.add_subparsers()

    # device name
    subparser = _sub_switch.add_parser("name", help="get the device name")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=device_name)

    # device presence
    subparser = _sub_switch.add_parser("present", help="get the device presence")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=device_presence)

    # device stats
    subparser = _sub_switch.add_parser("stats", help="get the device statistics")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=device_statistics)

    # blind
    subparser = _sub.add_parser("blind", help="Blind commands")
    _sub_switch = subparser.add_subparsers()

    # blind open
    subparser = _sub_switch.add_parser("open", help="open blinds")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=blind_set_open)

    # blind close
    subparser = _sub_switch.add_parser("close", help="close blinds")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=blind_set_close)

    # blind set level in percentage
    subparser = _sub_switch.add_parser(
        "set_level", help="set level of blinds in percentage (0=open)"
    )
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.add_argument(
        "level", type=float, metavar="LEVEL", help="blind level in percentage (0=open)"
    )
    subparser.set_defaults(func=blind_set_level_percentage)

    # thermostat
    subparser = _sub.add_parser("thermostat", help="Thermostat commands")
    _sub_switch = subparser.add_subparsers()

    # thermostat target temperature
    subparser = _sub_switch.add_parser(
        "set_target_temperature", help="set target temperature"
    )
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.add_argument(
        "temperature",
        type=float,
        metavar="TEMPERATURE",
        help="target temperature in (C)",
    )
    subparser.set_defaults(func=thermostat_set_target_temperature)

    # thermostat window_open
    subparser = _sub_switch.add_parser(
        "set_window_open", help="set window state to open"
    )
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.add_argument(
        "timespan",
        type=int,
        metavar="TIMESPAN",
        nargs="?",
        default=300,
        help="Open timespan in seconds (default=300s)",
    )
    subparser.set_defaults(func=thermostat_set_window_open)

    # thermostat boost_mpde
    subparser = _sub_switch.add_parser("set_boost_mode", help="activate the boost mode")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.add_argument(
        "timespan",
        type=int,
        choices=range(0, 86400),
        metavar="TIMESPAN",
        nargs="?",
        default=300,
        help="Boost timespan in seconds (default=300s)",
    )
    subparser.set_defaults(func=thermostat_set_boost_mode)

    # switch
    subparser = _sub.add_parser("switch", help="Switch commands")
    _sub_switch = subparser.add_subparsers()

    # switch get
    subparser = _sub_switch.add_parser("get", help="get state")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=switch_get)

    # switch on
    subparser = _sub_switch.add_parser("on", help="set on state")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=switch_on)

    # switch off
    subparser = _sub_switch.add_parser("off", help="set off state")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=switch_off)

    # switch toggle
    subparser = _sub_switch.add_parser("toggle", help="set off state")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=switch_toggle)

    # templates
    subparser = _sub.add_parser("template", help="Template commands")
    _sub_switch = subparser.add_subparsers()

    # list templates
    subparser = _sub_switch.add_parser("list", help="List all available templates")
    subparser.set_defaults(func=list_templates)

    # apply templates
    subparser = _sub_switch.add_parser("apply", help="Apply template")
    subparser.add_argument("ain", type=str, metavar="AIN", help="Actor Identification")
    subparser.set_defaults(func=template_apply)

    args = parser.parse_args(args)

    logging.basicConfig()
    if args.verbose:
        logging.getLogger("pyfritzhome").setLevel(logging.DEBUG)

    fritzbox = None
    try:
        fritzbox = Fritzhome(host=args.host, user=args.user, password=args.password)
        fritzbox.login()
        args.func(fritzbox, args)
    finally:
        if fritzbox is not None:
            fritzbox.logout()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceSwitch(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_get_switch_state(self):
        self.mock.side_effect = [
            Helper.response("switch/device_list"),
            "1",
            "0",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        assert device.get_switch_state()
        assert not device.get_switch_state()
        assert device.supported_features == [
            FritzhomeDeviceFeatures.POWER_METER,
            FritzhomeDeviceFeatures.TEMPERATURE,
            FritzhomeDeviceFeatures.SWITCH,
        ]
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "08761 0000434", "switchcmd": "getswitchstate", "sid": "0000001"},
        )

    def test_set_switch_state_toggle(self):
        self.mock.side_effect = [
            Helper.response("switch/device_list"),
            "1",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        device.set_switch_state_toggle()
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "08761 0000434", "switchcmd": "setswitchtoggle", "sid": "0000001"},
        )

    def test_set_switch_state_on(self):
        self.mock.side_effect = [
            Helper.response("switch/device_list"),
            "1",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        device.set_switch_state_on()
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "08761 0000434", "switchcmd": "setswitchon", "sid": "0000001"},
        )

    def test_set_switch_state_off(self):
        self.mock.side_effect = [
            Helper.response("switch/device_list"),
            "1",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        device.set_switch_state_off()
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "08761 0000434", "switchcmd": "setswitchoff", "sid": "0000001"},
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome


class TestFritzhomeDeviceRepeater(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"



"""The temperature device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceTemperature(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    offset = None
    temperature = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_temperature_sensor:
            self._update_temperature_from_node(node)

    # Temperature
    @property
    def has_temperature_sensor(self):
        """Check if the device has temperature function."""
        return self._has_feature(FritzhomeDeviceFeatures.TEMPERATURE)

    def _update_temperature_from_node(self, node):
        _LOGGER.debug("update temperature device")
        temperature_element = node.find("temperature")
        try:
            self.offset = (
                self.get_node_value_as_int(temperature_element, "offset") / 10.0
            )
        except ValueError:
            pass

        try:
            self.temperature = (
                self.get_node_value_as_int(temperature_element, "celsius") / 10.0
            )
        except ValueError:
            pass



"""The level device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceLevel(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    level = None
    levelpercentage = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_level:
            self._update_level_from_node(node)

    # Level
    @property
    def has_level(self):
        """Check if the device has level function."""
        return self._has_feature(FritzhomeDeviceFeatures.LEVEL)

    def _update_level_from_node(self, node):
        _LOGGER.debug("update level device")
        levelcontrol_element = node.find("levelcontrol")
        try:
            self.level = self.get_node_value_as_int(levelcontrol_element, "level")
            self.levelpercentage = self.get_node_value_as_int(
                levelcontrol_element, "levelpercentage"
            )
        except Exception:
            pass

    def get_level(self):
        """Get the level."""
        return self.level

    def get_level_percentage(self):
        """Get the level in percentage."""
        return self.levelpercentage

    def set_level(self, level):
        """Set the level."""
        self._fritz.set_level(self.ain, level)

    def set_level_percentage(self, levelpercentage):
        """Set the level in percentage."""
        self._fritz.set_level_percentage(self.ain, levelpercentage)



"""The button device class."""
# -*- coding: utf-8 -*-

import logging

from xml.etree import ElementTree
from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceButton(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_button:
            self._update_button_from_node(node)

    # Button
    @property
    def has_button(self):
        """Check if the device has button function."""
        return self._has_feature(FritzhomeDeviceFeatures.BUTTON)

    def _update_button_from_node(self, node):
        _LOGGER.debug("update button device")
        self.buttons = {}

        for element in node.findall("button"):
            button = FritzhomeButton(element)
            self.buttons[button.ain] = button

    def get_button_by_ain(self, ain):
        """Return the button by AIN."""
        return self.buttons[ain]


class FritzhomeButton(object):
    """The Fritzhome Button Device class."""

    ain = None
    identifier = None
    name = None
    last_pressed = None

    def __init__(self, node=None):
        """Create a button object."""
        if node is not None:
            self._update_from_node(node)

    def _update_from_node(self, node):
        _LOGGER.debug(ElementTree.tostring(node))
        self.ain = node.attrib["identifier"]
        self.identifier = node.attrib["id"]
        self.name = node.findtext("name")
        try:
            self.last_pressed = self.get_node_value_as_int(node, "lastpressedtimestamp")
        except ValueError:
            pass

    def get_node_value(self, elem, node):
        """Get the node value."""
        return elem.findtext(node)

    def get_node_value_as_int(self, elem, node) -> int:
        """Get the node value as integer."""
        return int(self.get_node_value(elem, node))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDevicePowermeter(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_get_switch_power(self):
        self.mock.side_effect = [
            Helper.response("powermeter/device_list"),
            "18000",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        assert device.get_switch_power() == 18000
        assert device.supported_features == [
            FritzhomeDeviceFeatures.POWER_METER,
            FritzhomeDeviceFeatures.TEMPERATURE,
            FritzhomeDeviceFeatures.SWITCH,
        ]
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "08761 0000434", "switchcmd": "getswitchpower", "sid": "0000001"},
        )

    def test_get_switch_energy(self):
        self.mock.side_effect = [
            Helper.response("powermeter/device_list"),
            "2000",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        assert device.get_switch_energy() == 2000
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "08761 0000434", "switchcmd": "getswitchenergy", "sid": "0000001"},
        )

    def test_get_switch_powermeter_properties(self):
        self.mock.side_effect = [
            Helper.response("powermeter/device_list"),
            "2000",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        assert device.energy == 707
        assert device.power == 1000
        assert device.voltage == 230000
        assert device.current == 4.3478260869565215

    def test_faulty_powermeter_properties(self):
        self.mock.side_effect = [
            Helper.response("powermeter/device_list_faulty"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")

        assert device.energy == 0
        assert device.power == 0
        assert device.voltage == 0
        assert device.current is None



"""The alarm device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceAlarm(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    alert_state = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_alarm:
            self._update_alarm_from_node(node)

    # Alarm
    @property
    def has_alarm(self):
        """Check if the device has alarm function."""
        return self._has_feature(FritzhomeDeviceFeatures.ALARM)

    def _update_alarm_from_node(self, node):
        _LOGGER.debug("update alert device")
        val = node.find("alert")
        try:
            self.alert_state = self.get_node_value_as_int_as_bool(val, "state")
        except (Exception, ValueError):
            self.alert_state = None



"""The light bulb device class."""
# -*- coding: utf-8 -*-

import logging

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceLightBulb(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    state = None
    hue = None
    saturation = None
    unmapped_hue = None
    unmapped_saturation = None
    color_temp = None
    color_mode = None
    supported_color_mode = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_lightbulb:
            self._update_lightbulb_from_node(node)

    # Light Bulb
    @property
    def has_lightbulb(self):
        """Check if the device has LightBulb function."""
        return self._has_feature(FritzhomeDeviceFeatures.LIGHTBULB)

    @property
    def has_color(self):
        """Check if the device has LightBulb function."""
        return self._has_feature(FritzhomeDeviceFeatures.COLOR)

    def _update_lightbulb_from_node(self, node):
        _LOGGER.debug("update light bulb device")
        state_element = node.find("simpleonoff")
        try:
            self.state = self.get_node_value_as_int_as_bool(state_element, "state")

        except ValueError:
            pass

        if self.has_color:
            colorcontrol_element = node.find("colorcontrol")
            try:
                self.color_mode = colorcontrol_element.attrib.get("current_mode")

                self.supported_color_mode = colorcontrol_element.attrib.get(
                    "supported_modes"
                )

            except ValueError:
                pass

            try:
                self.hue = self.get_node_value_as_int(colorcontrol_element, "hue")

                self.saturation = self.get_node_value_as_int(
                    colorcontrol_element, "saturation"
                )

                self.unmapped_hue = self.get_node_value_as_int(
                    colorcontrol_element, "unmapped_hue"
                )

                self.unmapped_saturation = self.get_node_value_as_int(
                    colorcontrol_element, "unmapped_saturation"
                )
            except ValueError:
                # reset values after color mode changed
                self.hue = None
                self.saturation = None
                self.unmapped_hue = None
                self.unmapped_saturation = None

            try:
                self.color_temp = self.get_node_value_as_int(
                    colorcontrol_element, "temperature"
                )

            except ValueError:
                # reset values after color mode changed
                self.color_temp = None

    def set_state_off(self):
        """Switch light bulb off."""
        self.state = True
        self._fritz.set_state_off(self.ain)

    def set_state_on(self):
        """Switch light bulb on."""
        self.state = True
        self._fritz.set_state_on(self.ain)

    def set_state_toggle(self):
        """Toogle light bulb state."""
        self.state = True
        self._fritz.set_state_toggle(self.ain)

    def get_colors(self):
        """Get the supported colors."""
        if self.has_color:
            return self._fritz.get_colors(self.ain)
        else:
            return {}

    def set_color(self, hsv, duration=0):
        """Set HSV color."""
        if self.has_color:
            self._fritz.set_color(self.ain, hsv, duration, True)

    def set_unmapped_color(self, hsv, duration=0):
        """Set unmapped HSV color (Free color selection)."""
        if self.has_color:
            self._fritz.set_color(self.ain, hsv, duration, False)

    def get_color_temps(self):
        """Get the supported color temperatures energy."""
        if self.has_color:
            return self._fritz.get_color_temps(self.ain)
        else:
            return []

    def set_color_temp(self, temperature, duration=0):
        """Set white color temperature."""
        if self.has_color:
            self._fritz.set_color_temp(self.ain, temperature, duration)



"""The feature list class."""
from enum import IntFlag


class FritzhomeDeviceFeatures(IntFlag):
    """The feature list class."""

    HANFUN_DEVICE = 0x0001  # bit 0
    LIGHTBULB = 0x0004  # bit 2
    ALARM = 0x0010  # bit 4
    BUTTON = 0x0020  # bit 5
    THERMOSTAT = 0x0040  # bit 6
    POWER_METER = 0x0080  # bit 7
    TEMPERATURE = 0x0100  # bit 8
    SWITCH = 0x0200  # bit 9
    DECT_REPEATER = 0x0400  # bit 10
    MICROPHONE = 0x0800  # bit 11
    HANFUN_UNIT = 0x2000  # bit 13
    SWITCHABLE = 0x8000  # bit 15
    LEVEL = 0x10000  # bit 16
    COLOR = 0x20000  # bit 17
    BLIND = 0x40000  # bit 18
    HUMIDITY = 0x100000  # bit 20



"""The thermostat device class."""
# -*- coding: utf-8 -*-

import logging
import time

from .fritzhomedevicebase import FritzhomeDeviceBase
from .fritzhomedevicefeatures import FritzhomeDeviceFeatures

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceThermostat(FritzhomeDeviceBase):
    """The Fritzhome Device class."""

    actual_temperature = None
    target_temperature = None
    eco_temperature = None
    comfort_temperature = None
    device_lock = None
    lock = None
    error_code = None
    window_open = None
    window_open_endtime = None
    boost_active = None
    boost_active_endtime = None
    adaptive_heating_active = None
    adaptive_heating_running = None
    summer_active = None
    holiday_active = None
    nextchange_endperiod = None
    nextchange_temperature = None

    def _update_from_node(self, node):
        super()._update_from_node(node)
        if self.present is False:
            return

        if self.has_thermostat:
            self._update_hkr_from_node(node)

    # Thermostat
    @property
    def has_thermostat(self):
        """Check if the device has thermostat function."""
        return self._has_feature(FritzhomeDeviceFeatures.THERMOSTAT)

    def _update_hkr_from_node(self, node):
        _LOGGER.debug("update thermostat device")
        hkr_element = node.find("hkr")

        try:
            self.actual_temperature = self.get_temp_from_node(hkr_element, "tist")
        except ValueError:
            pass

        self.target_temperature = self.get_temp_from_node(hkr_element, "tsoll")
        self.eco_temperature = self.get_temp_from_node(hkr_element, "absenk")
        self.comfort_temperature = self.get_temp_from_node(hkr_element, "komfort")

        # optional value
        try:
            self.device_lock = self.get_node_value_as_int_as_bool(
                hkr_element, "devicelock"
            )
            self.lock = self.get_node_value_as_int_as_bool(hkr_element, "lock")
            self.error_code = self.get_node_value_as_int(hkr_element, "errorcode")
            # keep battery values as fallback for Fritz!OS < 7.08

            if hkr_element.find("batterylow") is not None:
                self.battery_low = self.get_node_value_as_int_as_bool(
                    hkr_element, "batterylow"
                )
                self.battery_level = int(
                    self.get_node_value_as_int(hkr_element, "battery")
                )

            self.window_open = self.get_node_value_as_int_as_bool(
                hkr_element, "windowopenactiv"
            )
            self.window_open_endtime = (
                self.get_node_value_as_int(hkr_element, "windowopenactiveendtime")
                - time.time()
            )
            if self.window_open_endtime < 0:
                self.window_open_endtime = 0
            self.summer_active = self.get_node_value_as_int_as_bool(
                hkr_element, "summeractive"
            )
            self.holiday_active = self.get_node_value_as_int_as_bool(
                hkr_element, "holidayactive"
            )
            nextchange_element = hkr_element.find("nextchange")
            self.nextchange_endperiod = int(
                self.get_node_value_as_int(nextchange_element, "endperiod")
            )
            self.nextchange_temperature = self.get_temp_from_node(
                nextchange_element, "tchange"
            )

            if hkr_element.find("boostactive") is not None:
                self.boost_active = self.get_node_value_as_int_as_bool(
                    hkr_element, "boostactive"
                )
                self.boost_active_endtime = (
                    self.get_node_value_as_int(hkr_element, "boostactiveendtime")
                    - time.time()
                )
                if self.boost_active_endtime < 0:
                    self.boost_active_endtime = 0

            if hkr_element.find("adaptiveHeatingActive") is not None:
                self.adaptive_heating_active = self.get_node_value_as_int_as_bool(
                    hkr_element, "adaptiveHeatingActive"
                )
                self.adaptive_heating_running = self.get_node_value_as_int_as_bool(
                    hkr_element, "adaptiveHeatingRunning"
                )

        except Exception:
            pass

    def get_temperature(self):
        """Get the device temperature value."""
        return self._fritz.get_temperature(self.ain)

    def get_target_temperature(self):
        """Get the thermostate target temperature."""
        return self._fritz.get_target_temperature(self.ain)

    def set_target_temperature(self, temperature):
        """Set the thermostate target temperature."""
        return self._fritz.set_target_temperature(self.ain, temperature)

    def set_window_open(self, seconds):
        """Set the thermostate to window open."""
        return self._fritz.set_window_open(self.ain, seconds)

    def set_boost_mode(self, seconds):
        """Set the thermostate into boost mode."""
        return self._fritz.set_boost_mode(self.ain, seconds)

    def get_comfort_temperature(self):
        """Get the thermostate comfort temperature."""
        return self._fritz.get_comfort_temperature(self.ain)

    def get_eco_temperature(self):
        """Get the thermostate eco temperature."""
        return self._fritz.get_eco_temperature(self.ain)

    def get_hkr_state(self):
        """Get the thermostate state."""
        try:
            return {
                126.5: "off",
                127.0: "on",
                self.eco_temperature: "eco",
                self.comfort_temperature: "comfort",
            }[self.target_temperature]
        except KeyError:
            return "manual"

    def set_hkr_state(self, state):
        """Set the state of the thermostat.

        Possible values for state are: 'on', 'off', 'comfort', 'eco'.
        """
        try:
            value = {
                "off": 0,
                "on": 100,
                "eco": self.eco_temperature,
                "comfort": self.comfort_temperature,
            }[state]
        except KeyError:
            return

        self.set_target_temperature(value)



# -*- coding: utf-8 -*-

from requests.exceptions import ConnectionError
from unittest.mock import MagicMock, patch

import pytest

from pyfritzhome import Fritzhome, InvalidError, LoginError, NotLoggedInError

from .helper import Helper


class TestFritzhome(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "admin123")
        self.fritz._request = self.mock
        self.fritz._sid = "0000001"

    def test_login_fail(self):
        self.mock.side_effect = [
            Helper.response("login_rsp_without_valid_sid"),
            Helper.response("login_rsp_without_valid_sid"),
        ]

        with pytest.raises(LoginError) as ex:
            self.fritz.login()
        assert str(ex.value) == 'login for user="user" failed'

    def test_login_connection_error(self):
        self.mock.side_effect = ConnectionError

        with pytest.raises(ConnectionError):
            self.fritz.login()

    def test_login(self):
        self.mock.side_effect = [
            Helper.response("login_rsp_without_valid_sid"),
            Helper.response("login_rsp_with_valid_sid"),
        ]

        self.fritz.login()

    def test_login_pbkdf2(self):
        self.mock.side_effect = [
            Helper.response("login_rsp_without_valid_sid_pbkdf2"),
            Helper.response("login_rsp_with_valid_sid_pbkdf2"),
        ]
        """
        challenge was generated using
        http://home.mengelke.de/login_sid.lua?version=2 (Fritzbox Anmeldesimulator)
        response was generated using python code from
        https://avm.de/fileadmin/user_upload/Global/Service/Schnittstellen/AVM%20Technical%20Note%20-%20Session%20ID_deutsch%20-%20Nov2020.pdf
        the example challenge and response in this document is faulty and
        does not work with given example code
        """
        self.fritz.login()
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/login_sid.lua?version=2",
            {
                "username": "user",
                "response": "b9c232dea345233f5a893b2284931ac8$"
                "2825c7fbd8cdbcbaf93ca2e8d0798c31cf38394469a9ce89365778dc9103ad82",
            },
        )

    def test_logout(self):
        self.fritz.logout()
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/login_sid.lua",
            {"sid": "0000001", "security:command/logout": "1"},
        )

    def test_not_logged_in_error(self):
        self.fritz._sid = None
        with pytest.raises(NotLoggedInError) as ex:
            self.fritz.update_devices()
        assert str(ex.value) == "not logged in, login before doing any requests."

    def test_aha_request(self):
        self.fritz._aha_request(cmd="testcmd")
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "switchcmd": "testcmd"},
        )

        self.fritz._aha_request(cmd="testcmd", ain="1")
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "switchcmd": "testcmd", "ain": "1"},
        )

        self.fritz._aha_request(cmd="testcmd", ain="1", param={"a": "1", "b": "2"})
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "sid": "0000001",
                "switchcmd": "testcmd",
                "ain": "1",
                "a": "1",
                "b": "2",
            },
        )

    def test_aha_request_invalid(self):
        self.mock.side_effect = [
            "inval",
        ]

        with pytest.raises(InvalidError):
            self.fritz._aha_request(cmd="estcmd")

    def test_get_device_element(self):
        self.mock.side_effect = [
            Helper.response("base/device_list"),
            Helper.response("base/device_list"),
            Helper.response("base/device_list"),
        ]

        element = self.fritz.get_device_element("08761 0000434")
        assert element.attrib["identifier"] == "08761 0000434"
        assert element.attrib["fwversion"] == "03.33"

        element = self.fritz.get_device_element("08761 1048079")
        assert element.attrib["identifier"] == "08761 1048079"
        assert element.attrib["fwversion"] == "03.44"

        element = self.fritz.get_device_element("unknown")
        assert element is None

    def test_get_device_by_ain(self):
        self.mock.side_effect = [
            Helper.response("base/device_list"),
            Helper.response("base/device_list"),
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("08761 0000434")
        assert device.ain == "08761 0000434"

    def test_aha_get_devices(self):
        self.mock.side_effect = [
            Helper.response("base/device_list"),
        ]
        self.fritz.update_devices()

        devices = self.fritz.get_devices()
        assert devices[0].name == "Steckdose"
        assert devices[1].name == "FRITZ!DECT Rep 100 #1"

        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "switchcmd": "getdevicelistinfos"},
        )

    def test_get_device_name(self):
        self.mock.side_effect = ["testname"]

        assert self.fritz.get_device_name(ain="1234") == "testname"

        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "ain": "1234", "switchcmd": "getswitchname"},
        )

    def test_get_template_by_ain(self):
        self.mock.side_effect = [
            Helper.response("templates/template_list"),
            Helper.response("templates/template_list"),
        ]

        self.fritz.update_templates()
        device = self.fritz.get_template_by_ain("tmp0B32F7-1B0650682")
        assert device.ain == "tmp0B32F7-1B0650682"

    def test_aha_get_templates(self):
        self.mock.side_effect = [
            Helper.response("templates/template_list"),
        ]
        self.fritz.update_templates()

        templates = self.fritz.get_templates()
        assert templates[0].name == "Base Data"
        assert templates[1].name == "One Device"

        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "switchcmd": "gettemplatelistinfos"},
        )

    def test_aha_apply_template(self):
        self.fritz.apply_template("1234")

        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "ain": "1234", "switchcmd": "applytemplate"},
        )

    def test_set_target_temperature(self):
        self.fritz.set_target_temperature("1", 25.5)
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "ain": "1", "switchcmd": "sethkrtsoll", "param": 51},
        )

        self.fritz.set_target_temperature("1", 0.0)
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "ain": "1", "switchcmd": "sethkrtsoll", "param": 253},
        )

        self.fritz.set_target_temperature("1", 32.0)
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"sid": "0000001", "ain": "1", "switchcmd": "sethkrtsoll", "param": 254},
        )

    @patch("time.time", MagicMock(return_value=1000))
    def test_set_window_open(self):
        self.fritz.set_window_open("1", 25)
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "sid": "0000001",
                "ain": "1",
                "switchcmd": "sethkrwindowopen",
                "endtimestamp": 1000 + 25,
            },
        )

    @patch("time.time", MagicMock(return_value=1000))
    def test_set_boost_mode(self):
        self.fritz.set_boost_mode("1", 25)
        self.fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "sid": "0000001",
                "ain": "1",
                "switchcmd": "sethkrboost",
                "endtimestamp": 1000 + 25,
            },
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceTemperature(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_get_temperature(self):
        self.mock.side_effect = [
            Helper.response("thermostat/device_hkr_fritzos_7"),
            "245",
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("12345")

        assert device.get_temperature() == 24.5
        assert device.supported_features == [
            FritzhomeDeviceFeatures.THERMOSTAT,
            FritzhomeDeviceFeatures.TEMPERATURE,
        ]
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {"ain": "12345", "switchcmd": "gettemperature", "sid": "0000001"},
        )



"""The base device class."""
# -*- coding: utf-8 -*-

from __future__ import print_function


import logging

from pyfritzhome.devicetypes.fritzhomeentitybase import FritzhomeEntityBase

_LOGGER = logging.getLogger(__name__)


class FritzhomeDeviceBase(FritzhomeEntityBase):
    """The Fritzhome Device class."""

    battery_level = None
    battery_low = None
    identifier = None
    is_group = None
    fw_version = None
    group_members = None
    manufacturer = None
    productname = None
    present = None
    tx_busy = None

    def __repr__(self):
        """Return a string."""
        return "{ain} {identifier} {manuf} {prod} {name}".format(
            ain=self.ain,
            identifier=self.identifier,
            manuf=self.manufacturer,
            prod=self.productname,
            name=self.name,
        )

    def update(self):
        """Update the device values."""
        self._fritz.update_devices()

    def _update_from_node(self, node):
        _LOGGER.debug("update base device")
        super()._update_from_node(node)
        self.ain = node.attrib["identifier"]
        self.identifier = node.attrib["id"]
        self.fw_version = node.attrib["fwversion"]
        self.manufacturer = node.attrib["manufacturer"]
        self.productname = node.attrib["productname"]

        self.present = bool(int(node.findtext("present")))

        groupinfo = node.find("groupinfo")
        self.is_group = groupinfo is not None
        if self.is_group:
            self.group_members = str(groupinfo.findtext("members")).split(",")

        try:
            self.tx_busy = self.get_node_value_as_int_as_bool(node, "txbusy")
        except Exception:
            pass

        try:
            self.battery_low = self.get_node_value_as_int_as_bool(node, "batterylow")
            self.battery_level = int(self.get_node_value_as_int(node, "battery"))
        except Exception:
            pass

    # General
    def get_present(self):
        """Check if the device is present."""
        return self._fritz.get_device_present(self.ain)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeDeviceBlind(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

    def test_device_response(self):
        self.mock.side_effect = [
            Helper.response("blind/device_blind_rollotron1213"),
        ]

        self.fritz.update_devices()
        device1 = self.fritz.get_device_by_ain("14276 1234567")
        assert device1.present
        assert not device1.tx_busy
        assert device1.supported_features == [FritzhomeDeviceFeatures.HANFUN_DEVICE]

        device2 = self.fritz.get_device_by_ain("14276 1234567-1")
        assert device2.present
        assert not device2.tx_busy
        assert device2.endpositionsset
        assert device2.supported_features == [
            FritzhomeDeviceFeatures.ALARM,
            FritzhomeDeviceFeatures.HANFUN_UNIT,
            FritzhomeDeviceFeatures.LEVEL,
            FritzhomeDeviceFeatures.BLIND,
        ]

        assert device2.level == 252
        assert device2.levelpercentage == 99

        assert device2.get_level() == 252
        assert device2.get_level_percentage() == 99

    def test_set_level(self):
        self.mock.side_effect = [
            Helper.response("blind/device_blind_rollotron1213"),
            None,
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("14276 1234567-1")

        device.set_level(100)
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "switchcmd": "setlevel",
                "sid": "0000001",
                "ain": "14276 1234567-1",
                "level": 100,
            },
        )

    def test_set_level_percentage(self):
        self.mock.side_effect = [
            Helper.response("blind/device_blind_rollotron1213"),
            None,
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("14276 1234567-1")

        device.set_level_percentage(50)
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "switchcmd": "setlevelpercentage",
                "sid": "0000001",
                "ain": "14276 1234567-1",
                "level": 50,
            },
        )

    def test_set_blind_open(self):
        self.mock.side_effect = [
            Helper.response("blind/device_blind_rollotron1213"),
            None,
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("14276 1234567-1")

        device.set_blind_open()
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "switchcmd": "setblind",
                "sid": "0000001",
                "ain": "14276 1234567-1",
                "target": "open",
            },
        )

    def test_set_blind_close(self):
        self.mock.side_effect = [
            Helper.response("blind/device_blind_rollotron1213"),
            None,
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("14276 1234567-1")

        device.set_blind_close()
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "switchcmd": "setblind",
                "sid": "0000001",
                "ain": "14276 1234567-1",
                "target": "close",
            },
        )

    def test_set_blind_stop(self):
        self.mock.side_effect = [
            Helper.response("blind/device_blind_rollotron1213"),
            None,
        ]

        self.fritz.update_devices()
        device = self.fritz.get_device_by_ain("14276 1234567-1")

        device.set_blind_stop()
        device._fritz._request.assert_called_with(
            "http://10.0.0.1/webservices/homeautoswitch.lua",
            {
                "switchcmd": "setblind",
                "sid": "0000001",
                "ain": "14276 1234567-1",
                "target": "stop",
            },
        )



"""Init file for the device types."""

from .fritzhomedevicealarm import FritzhomeDeviceAlarm
from .fritzhomedevicebutton import FritzhomeDeviceButton
from .fritzhomedevicehumidity import FritzhomeDeviceHumidity
from .fritzhomedevicelevel import FritzhomeDeviceLevel
from .fritzhomedevicepowermeter import FritzhomeDevicePowermeter
from .fritzhomedevicerepeater import FritzhomeDeviceRepeater
from .fritzhomedeviceswitch import FritzhomeDeviceSwitch
from .fritzhomedevicetemperature import FritzhomeDeviceTemperature
from .fritzhomedevicethermostat import FritzhomeDeviceThermostat
from .fritzhomedevicelightbulb import FritzhomeDeviceLightBulb
from .fritzhomedeviceblind import FritzhomeDeviceBlind
from .fritzhometemplate import FritzhomeTemplate


__all__ = (
    "FritzhomeDeviceAlarm",
    "FritzhomeDeviceButton",
    "FritzhomeDeviceHumidity",
    "FritzhomeDeviceLevel",
    "FritzhomeDevicePowermeter",
    "FritzhomeDeviceRepeater",
    "FritzhomeDeviceSwitch",
    "FritzhomeDeviceTemperature",
    "FritzhomeDeviceThermostat",
    "FritzhomeDeviceLightBulb",
    "FritzhomeDeviceBlind",
    "FritzhomeTemplate",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest.mock import MagicMock

from pyfritzhome import Fritzhome
from pyfritzhome.devicetypes.fritzhomedevicefeatures import FritzhomeDeviceFeatures

from .helper import Helper


class TestFritzhomeTemplate(object):
    def setup_method(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome("10.0.0.1", "user", "pass")
        self.fritz._request = self.mock
        self.fritz._devices = {}
        self.fritz._sid = "0000001"

        self.mock.side_effect = [Helper.response("templates/template_list")]

        self.fritz.update_templates()

    def test_template_init(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B0650682")

        assert template.ain == "tmp0B32F7-1B0650682"
        assert template._functionsbitmask == 320
        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper
        assert template.supported_features == [
            FritzhomeDeviceFeatures.THERMOSTAT,
            FritzhomeDeviceFeatures.TEMPERATURE,
        ]

    def test_template_removed(self):
        self.mock.side_effect = [
            Helper.response("templates/template_list"),
            Helper.response("templates/template_list_removed_template"),
            Helper.response("templates/template_list_removed_template"),
        ]

        self.fritz.update_templates()
        assert len(self.fritz.get_templates()) == 12
        self.fritz.update_templates()
        assert len(self.fritz.get_templates()) == 12
        self.fritz.update_templates(ignore_removed=False)
        assert len(self.fritz.get_templates()) == 11

    def test_template_with_single_device(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B0650234")

        assert template.devices == ["08735 0525249"]

    def test_template_with_multiple_devices(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1C40A2B8A")

        # fmt: off
        expected_devices = set(["08735 0525249",
                                "08735 0525249",
                                "08735 0340143",
                                "08735 0526125"])
        # fmt: on
        assert len(expected_devices.intersection(template.devices)) == len(
            expected_devices
        )

    def test_template_applies_hkr_summer(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA20")

        assert template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_hkr_temperature(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA21")

        assert not template.apply_hkr_summer
        assert template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_hkr_holidays(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA22")

        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_hkr_time_table(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA23")

        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_relay_manual(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA24")

        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_relay_automatic(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA25")

        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_level(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA26")

        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert template.apply_level
        assert not template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_color(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA27")

        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert template.apply_color
        assert not template.apply_dialhelper

    def test_template_applies_dialhelper(self):
        template = self.fritz.get_template_by_ain("tmp0B32F7-1B064FA28")

        assert not template.apply_hkr_summer
        assert not template.apply_hkr_temperature
        assert not template.apply_hkr_holidays
        assert not template.apply_hkr_time_table
        assert not template.apply_relay_manual
        assert not template.apply_relay_automatic
        assert not template.apply_level
        assert not template.apply_color
        assert template.apply_dialhelper

    def test_has_template(self):
        self.mock.side_effect = ["invalid_xml"]
        assert not self.fritz.has_templates()
